@@ -1,7 +1,6 @@
 package client_test
 
 import (
-	"os"
 	"sync"
 	"testing"
 
@@ -15,48 +14,69 @@ func TestInitApp(t *testing.T) {
 	t.Parallel()
 
 	a := client.New()
+
 	err := a.Run()
 	require.Error(t, err, "Run should return usage")
 }
 
 func TestAppHelp(t *testing.T) {
+	t.Parallel()
+
 	a := client.New()
+	a.SetArgs("--help")
 
-	defer changeArgs("adsysctl", "--help")()
 	err := a.Run()
 	require.NoError(t, err, "Run should return no error")
 }
 
 func TestAppCompletion(t *testing.T) {
+	t.Parallel()
+
 	a := client.New()
+	a.SetArgs("completion", "bash")
 
-	defer changeArgs("adsysctl", "completion", "bash")()
 	err := a.Run()
 	require.NoError(t, err, "Completion should not use socket and always be reachable")
 }
 
 func TestAppNoUsageError(t *testing.T) {
-	a := client.New()
+	t.Parallel()
 
-	defer changeArgs("adsysctl", "completion", "bash")()
-	a.Run()
+	a := client.New()
+	a.SetArgs("completion", "bash")
+
+	err := a.Run()
+	require.NoError(t, err, "Completion should not return an error")
 	isUsageError := a.UsageError()
 	require.False(t, isUsageError, "No usage error is reported as such")
 }
 
 func TestAppUsageError(t *testing.T) {
-	a := client.New()
+	t.Parallel()
 
-	defer changeArgs("adsysctl", "doesnotexist")()
-	a.Run()
+	a := client.New()
+	a.SetArgs("doesnotexist")
+
+	err := a.Run()
+	require.Error(t, err, "Run should return usage")
 	isUsageError := a.UsageError()
 	require.True(t, isUsageError, "Usage error is reported as such")
 }
 
+func TestAppUsageErrorReportsSuggestions(t *testing.T) {
+	a := client.New()
+	a.SetArgs("hel")
+
+	err := a.Run()
+	require.ErrorContains(t, err, "Did you mean this?\n\thelp", "Run should return usage with suggestions")
+}
+
 func TestAppCanQuitWhenExecute(t *testing.T) {
+	t.Parallel()
+
 	a := client.New()
 	a.AddWaitCommand()
-	defer changeArgs("adsysctl", "wait")()
+	a.SetArgs("wait")
 
 	wg := sync.WaitGroup{}
 	wg.Add(1)
@@ -72,23 +92,29 @@ func TestAppCanQuitWhenExecute(t *testing.T) {
 }
 
 func TestAppCanQuitAfterExecute(t *testing.T) {
-	a := client.New()
+	t.Parallel()
 
-	defer changeArgs("adsysctl", "completion", "bash")()
-	a.Run()
+	a := client.New()
+	a.SetArgs("completion", "bash")
+
+	err := a.Run()
+	require.NoError(t, err, "Run should return no error")
 	a.Quit()
 }
 
 func TestAppCanQuitWithoutExecute(t *testing.T) {
+	t.Parallel()
+
 	a := client.New()
 	a.Quit()
 }
 
 func TestAppCanSigHupWhenExecute(t *testing.T) {
-	a := client.New()
+	t.Parallel()
 
+	a := client.New()
 	a.AddWaitCommand()
-	defer changeArgs("adsysctl", "wait")()
+	a.SetArgs("wait")
 
 	wg := sync.WaitGroup{}
 	wg.Add(1)
@@ -104,10 +130,13 @@ func TestAppCanSigHupWhenExecute(t *testing.T) {
 }
 
 func TestAppCanSigHupAfterExecute(t *testing.T) {
-	a := client.New()
+	t.Parallel()
 
-	defer changeArgs("adsysctl", "completion", "bash")()
-	a.Run()
+	a := client.New()
+	a.SetArgs("completion", "bash")
+
+	err := a.Run()
+	require.NoError(t, err, "Run should return no error")
 	require.True(t, a.Hup(), "Hup returns true for client")
 }
 
@@ -119,10 +148,3 @@ func TestAppGetRootCmd(t *testing.T) {
 }
 
 // TODO: config change
-
-// changeArgs allows changing command line arguments and return a function to return it
-func changeArgs(args ...string) func() {
-	orig := os.Args
-	os.Args = args
-	return func() { os.Args = orig }
-}
