@@ -6,6 +6,7 @@ import (
 
 	"github.com/fatih/color"
 	"github.com/stretchr/testify/require"
+	"github.com/ubuntu/adsys/internal/testutils"
 )
 
 func TestColorizePolicies(t *testing.T) {
@@ -47,8 +48,6 @@ Policies from user configuration:
 	got, err := colorizePolicies(policies)
 	require.NoError(t, err, "colorizePolicies should not return an error")
 
-	want, err := os.ReadFile("testdata/golden/colorize.golden")
-	require.NoError(t, err, "Setup: failed to read colorized golden file")
-
-	require.Equal(t, string(want), got, "colorizePolicies returned expected formatted output")
+	want := testutils.LoadWithUpdateFromGolden(t, got)
+	require.Equal(t, want, got, "colorizePolicies returned expected formatted output")
 }
