@@ -1,14 +1,15 @@
 package client
 
 import (
+	"errors"
 	"fmt"
 	"io"
 
+	"github.com/leonelquinteros/gotext"
 	"github.com/spf13/cobra"
 	"github.com/ubuntu/adsys"
 	"github.com/ubuntu/adsys/internal/adsysservice"
 	"github.com/ubuntu/adsys/internal/cmdhandler"
-	"github.com/ubuntu/adsys/internal/i18n"
 	"google.golang.org/grpc/codes"
 	"google.golang.org/grpc/status"
 )
@@ -16,7 +17,7 @@ import (
 func (a *App) installService() {
 	mainCmd := &cobra.Command{
 		Use:   "service COMMAND",
-		Short: i18n.G("Service management"),
+		Short: gotext.Get("Service management"),
 		Args:  cmdhandler.SubcommandsRequiredWithSuggestions,
 		RunE:  cmdhandler.NoCmd,
 	}
@@ -24,31 +25,31 @@ func (a *App) installService() {
 
 	cmd := &cobra.Command{
 		Use:               "cat",
-		Short:             i18n.G("Print service logs"),
+		Short:             gotext.Get("Print service logs"),
 		Args:              cobra.NoArgs,
 		ValidArgsFunction: cmdhandler.NoValidArgs,
-		RunE:              func(cmd *cobra.Command, args []string) error { return a.serviceCat() },
+		RunE:              func(_ *cobra.Command, _ []string) error { return a.serviceCat() },
 	}
 	mainCmd.AddCommand(cmd)
 
 	cmd = &cobra.Command{
 		Use:               "status",
-		Short:             i18n.G("Print service status"),
+		Short:             gotext.Get("Print service status"),
 		Args:              cobra.NoArgs,
 		ValidArgsFunction: cmdhandler.NoValidArgs,
-		RunE:              func(cmd *cobra.Command, args []string) error { return a.getStatus() },
+		RunE:              func(_ *cobra.Command, _ []string) error { return a.getStatus() },
 	}
 	mainCmd.AddCommand(cmd)
 
 	var stopForce *bool
 	cmd = &cobra.Command{
 		Use:               "stop",
-		Short:             i18n.G("Requests to stop the service once all connections are done"),
+		Short:             gotext.Get("Requests to stop the service once all connections are done"),
 		Args:              cobra.NoArgs,
 		ValidArgsFunction: cmdhandler.NoValidArgs,
-		RunE:              func(cmd *cobra.Command, args []string) error { return a.serviceStop(*stopForce) },
+		RunE:              func(_ *cobra.Command, _ []string) error { return a.serviceStop(*stopForce) },
 	}
-	stopForce = cmd.Flags().BoolP("force", "f", false, i18n.G("force will shut it down immediately and drop existing connections."))
+	stopForce = cmd.Flags().BoolP("force", "f", false, gotext.Get("force will shut it down immediately and drop existing connections."))
 	mainCmd.AddCommand(cmd)
 }
 
@@ -68,7 +69,7 @@ func (a *App) serviceCat() error {
 	for {
 		msg, err := stream.Recv()
 		if err != nil {
-			if err == io.EOF {
+			if errors.Is(err, io.EOF) {
 				break
 			}
 			return err
@@ -114,9 +115,12 @@ func (a *App) serviceStop(force bool) error {
 		return err
 	}
 
-	if _, err := stream.Recv(); err != nil && err != io.EOF {
+	if _, err := stream.Recv(); err != nil {
 		// Ignore "transport is closing" error if force (i.e immediately drop all connections) was used.
-		if force && status.Code(err) == codes.Unavailable {
+		// EOF check is done on the same line to avoid test coverage drift when the Unavailable error is not triggered,
+		// indicating that the underlying transport is already closed before we could get a response.
+		// The error is harmless as it indicates that the server has already stopped.
+		if (force && status.Code(err) == codes.Unavailable) || errors.Is(err, io.EOF) {
 			return nil
 		}
 		return err
