@@ -3,12 +3,15 @@ package adsysservice
 
 import (
 	"context"
+	"errors"
 	"fmt"
+	"math"
 	"os"
 	"strings"
 	"time"
 
 	"github.com/godbus/dbus/v5"
+	"github.com/leonelquinteros/gotext"
 	"github.com/sirupsen/logrus"
 	"github.com/ubuntu/adsys"
 	"github.com/ubuntu/adsys/internal/ad"
@@ -22,7 +25,6 @@ import (
 	"github.com/ubuntu/adsys/internal/grpc/interceptorschain"
 	"github.com/ubuntu/adsys/internal/grpc/logconnections"
 	log "github.com/ubuntu/adsys/internal/grpc/logstreamer"
-	"github.com/ubuntu/adsys/internal/i18n"
 	"github.com/ubuntu/adsys/internal/policies"
 	"github.com/ubuntu/decorate"
 	"google.golang.org/grpc"
@@ -69,6 +71,7 @@ type options struct {
 	systemUnitDir  string
 	globalTrustDir string
 	adBackend      string
+	gpoListTimeout time.Duration
 	sssConfig      sss.Config
 	winbindConfig  winbind.Config
 	authorizer     authorizerer
@@ -185,11 +188,19 @@ func WithWinbindConfig(c winbind.Config) func(o *options) error {
 	}
 }
 
+// WithGpoListTimeout specifies the timeout for the gpo list.
+func WithGpoListTimeout(t time.Duration) func(o *options) error {
+	return func(o *options) error {
+		o.gpoListTimeout = t
+		return nil
+	}
+}
+
 // New returns a new instance of an AD service.
 // If url or domain is empty, we load the missing parameters from sssd.conf, taking first
 // domain in the list if not provided.
 func New(ctx context.Context, opts ...option) (s *Service, err error) {
-	defer decorate.OnError(&err, i18n.G("couldn't create adsys service"))
+	defer decorate.OnError(&err, gotext.Get("couldn't create adsys service"))
 
 	// defaults
 	args := options{}
@@ -205,7 +216,7 @@ func New(ctx context.Context, opts ...option) (s *Service, err error) {
 	if runDir == "" {
 		runDir = consts.DefaultRunDir
 	}
-	// #nosec G301 - we need to ensure users have access directly to their own scripts
+	//nolint:gosec // G301 - we need to ensure users have access directly to their own scripts
 	if err := os.MkdirAll(runDir, 0755); err != nil {
 		return nil, err
 	}
@@ -240,6 +251,8 @@ func New(ctx context.Context, opts ...option) (s *Service, err error) {
 		adOptions = append(adOptions, ad.WithRunDir(args.runDir))
 	}
 
+	adOptions = append(adOptions, ad.WithGpoListTimeout(args.gpoListTimeout))
+
 	hostname, err := os.Hostname()
 	if err != nil {
 		return nil, err
@@ -261,7 +274,7 @@ func New(ctx context.Context, opts ...option) (s *Service, err error) {
 		adBackend, err = winbind.New(ctx, args.winbindConfig, hostname)
 	}
 	if err != nil {
-		return nil, fmt.Errorf(i18n.G("could not initialize AD backend: %v"), err)
+		return nil, errors.New(gotext.Get("could not initialize AD backend: %v", err))
 	}
 
 	adc, err := ad.New(ctx, adBackend, hostname, adOptions...)
@@ -354,7 +367,7 @@ func (s *Service) RegisterGRPCServer(d *daemon.Daemon) *grpc.Server {
 // Quit cleans every ressources than the service was using.
 func (s *Service) Quit(ctx context.Context) {
 	if err := s.bus.Close(); err != nil {
-		log.Warningf(ctx, i18n.G("Can't disconnect system dbus: %v"), err)
+		log.Warning(ctx, gotext.Get("Can't disconnect system dbus: %v", err))
 	}
 }
 
@@ -371,7 +384,13 @@ func initSystemTime(bus *dbus.Conn) *time.Time {
 		log.Warningf(context.Background(), "invalid next system startup time: %v", val.Value())
 		return nil
 	}
+	start = start / 1000000
+	if start > math.MaxInt64 {
+		log.Warningf(context.Background(), "next system startup time should be int64: %d", start)
+		return nil
+	}
 
-	initSystemTime := time.Unix(int64(start)/1000000, 0)
+	//nolint:gosec // We handle the overflow check above.
+	initSystemTime := time.Unix(int64(start), 0)
 	return &initSystemTime
 }
