@@ -5,9 +5,9 @@ import (
 	"fmt"
 	"strings"
 
+	"github.com/leonelquinteros/gotext"
 	"github.com/spf13/cobra"
 	"github.com/spf13/viper"
-	"github.com/ubuntu/adsys/internal/i18n"
 	"github.com/ubuntu/decorate"
 )
 
@@ -18,7 +18,7 @@ func NoCmd(_ *cobra.Command, _ []string) error {
 
 // ZeroOrNArgs returns an error if there are not 0 or exactly N arguments for the given command.
 func ZeroOrNArgs(n int) cobra.PositionalArgs {
-	return func(cmd *cobra.Command, args []string) error {
+	return func(_ *cobra.Command, args []string) error {
 		if len(args) != 0 && len(args) != n {
 			return fmt.Errorf("requires either no arguments or exactly %d, only received %d", n, len(args))
 		}
@@ -35,7 +35,7 @@ func NoValidArgs(_ *cobra.Command, _ []string, _ string) ([]string, cobra.ShellC
 // README and manpage refers to them in each subsection (parents are differents, but only one is kept if we use the same object).
 func RegisterAlias(cmd, parent *cobra.Command) {
 	alias := *cmd
-	t := fmt.Sprintf(i18n.G("Alias of %q"), cmd.CommandPath())
+	t := gotext.Get("Alias of %q", cmd.CommandPath())
 	if alias.Long != "" {
 		t = fmt.Sprintf("%s (%s)", alias.Long, t)
 	}
@@ -45,15 +45,17 @@ func RegisterAlias(cmd, parent *cobra.Command) {
 
 // InstallVerboseFlag adds the -v and -vv options and returns the reference to it.
 func InstallVerboseFlag(cmd *cobra.Command, viper *viper.Viper) *int {
-	r := cmd.PersistentFlags().CountP("verbose", "v", i18n.G("issue INFO (-v), DEBUG (-vv) or DEBUG with caller (-vvv) output"))
-	decorate.LogOnError(viper.BindPFlag("verbose", cmd.PersistentFlags().Lookup("verbose")))
+	r := cmd.PersistentFlags().CountP("verbose", "v", gotext.Get("issue INFO (-v), DEBUG (-vv) or DEBUG with caller (-vvv) output"))
+	err := viper.BindPFlag("verbose", cmd.PersistentFlags().Lookup("verbose"))
+	decorate.LogOnError(&err)
 	return r
 }
 
 // InstallSocketFlag adds the -s and --sockets options and returns the reference to it.
 func InstallSocketFlag(cmd *cobra.Command, viper *viper.Viper, defaultPath string) *string {
-	s := cmd.PersistentFlags().StringP("socket", "s", defaultPath, i18n.G("socket path to use between daemon and client. Can be overridden by systemd socket activation."))
-	decorate.LogOnError(viper.BindPFlag("socket", cmd.PersistentFlags().Lookup("socket")))
+	s := cmd.PersistentFlags().StringP("socket", "s", defaultPath, gotext.Get("socket path to use between daemon and client. Can be overridden by systemd socket activation."))
+	err := viper.BindPFlag("socket", cmd.PersistentFlags().Lookup("socket"))
+	decorate.LogOnError(&err)
 	return s
 }
 
@@ -63,7 +65,7 @@ func InstallConfigFlag(cmd *cobra.Command, persistent bool) *string {
 	if persistent {
 		target = cmd.PersistentFlags()
 	}
-	return target.StringP("config", "c", "", i18n.G("use a specific configuration file"))
+	return target.StringP("config", "c", "", gotext.Get("use a specific configuration file"))
 }
 
 // CalledCmd returns the actual command called by the user inferred from the arguments.
